/*
 * Simple bc Widget
 * URL: http://www.3thirty.net/simple-bc
 * Author: Ethan Smith (ethan@3thirty.net)
 * Created: 22/10/08
 * 
 * Simple dashborad calculator utilising the baisc calculator, bc. This is really just an interface that
 * passes exactly what you enter to the bc command and prints out the result. The idea behind this widget
 * is to make it really quick and easy to perform simple calculations.
 *
 * Some of the things that this widget does (aka why this is better than the commandline):
 *		- Typing anywhere on the widget will start typing in the sum box
 *		- Pressing enter will perform the calculation
 *		- Pressing escape will clear the textfield
 *		- Starting a calculation with an arithmetic operator (+-/*) will perform the calculation against
 *		  the previous result
 *		- The "previous result variable" allows you to substitute the previous result at any point in
 *		  the current calculation (note that this can be a single letter or a longer word, but you'll have
 *		  all sorts of problems if you try to use a number!)
 *
 * Some of the bc features that are known to work:
 *		- Enter almost anything and bc will do it's best to make a calculation out of it
 *					Example "xyz + 14 - ff * 2 - (v + c)" = 14
 *		- multiline calculations are supported, with a semicolon between each calculation. The results
 *		  will also be separated by semicolons.
 *					Example: "10+4; 11+5;" = 14;15
 *		- variable assignment works. Just be careful that variable names don't get trodden on by the
 *		  "previous result variable".
 *					Example: "x=12; x=x+48; x-10" = 50
 *
 * Notes on the implementation of bc:
 *		- this widget really just passes through the calculation almost exactly as entered. Try running 
 *		  bc from the commandline, see "man bc" or http://en.wikipedia.org/wiki/Bc_(Unix) for more
 *		- the scale (number of decimal places) is set to 3 by the widget
 */ 

// global reference to the system object. We can assume bc is open and available
// for stdin/out operations at all times
var prevResult;

// read in the magicLetter/previous result variable. Default to "x" if it's not set
var magicLetter = widget.preferenceForKey("magicLetter");
if (magicLetter == "")
	magicLetter = "x";

var scale = 3; // the number of decimal places

//
// Function: load()
// Called by HTML body element's onload event when the widget is ready to start
//
function load()
{
    setupParts();
	
	// make the textbox look pretty
	document.getElementById("sum").style.background = "transparent";
	document.getElementById("sum").style.border = "1px #c0c0c0 solid";
	document.getElementById("sum").style.cursor = "pointer";
	document.getElementById("sum").onfocus = function (){
		this.value = ""
	}
	
	document.onkeydown = keyHandler;
	
	// make the equals sign do the calculation (or at least look like it does; once
	// the textfield blurs, it'll do it anyway)
	document.getElementById("equalsSign").style.cursor = "pointer";
	
	// link up the brag tag
	document.getElementById("bragTag").style.cursor = "pointer";
	document.getElementById("bragTag").onclick = function (){
		widget.openURL("http://www.3thirty.net/simple-bc");
	}
}

/* 
 * do some friendly stuff with the keyboard:
 *	- enter will evaluate the sum in the box
 *	- escape will clear the sum
 *	- any other key presses will automatically move us into the sum box if we're not already there
 *
 * @param	e	The keydown event
 */
function keyHandler (e){
	if (e.which == 13) // enter key
		stdIn();
	else if (e.which == 27) // esc key
		document.getElementById("sum").value = "";
	else // anything else
		document.getElementById("sum").focus();
}

/*
 * pass a calculation to bc via the commandline. Get the sum from the textbox. We'll let bc
 * deal with crappy input, but will replace the magic letter with the previous result
 */
function stdIn(){
	value = document.getElementById("sum").value;
	
	// allow users to just enter a sum like "+4". Prepend the previous result
	if (value.match(/^[\+\-\*\%\/]/))
		value = magicLetter + value;
	
	// match the magic letter so long as it's not part of a word. This will let us use variables more easily
	var magicLetterRegex = eval("/([^a-zA-Z])*" + magicLetter + "([^a-zA-Z])/");
	alert("magic letter regex: " + magicLetterRegex);
	value = value.replace(magicLetterRegex, "$1" + prevResult + "$2");

	alert("Value being passed to bc: " + value);
	if (value != "")
		widget.system("echo \"scale=" + scale + ";" + value + "\" | /usr/bin/bc -q", stdOut);
}

/*
 * read the response from bc
 * @param	bc	a copy of the bc system object
 */
function stdOut(
	bc
){
	if (typeof bc == "object" && bc.outputString != "undefined" && bc.outputString){
		// handle multi-line responses -- separate the results with a semicolon
		var output = bc.outputString.replace(/\n/g, ";");
		output = output.replace(/;$/, ""); // trim the trailing semicolon in all cases
		
		// if it's a round number, don't show the decimals
		output = output.replace(/(\.[1-9]*)0+$/, "$1");
		output = output.replace(/\.$/, ""); // trim the trailing . if there is one
		
		document.getElementById("result").innerHTML = output;
		prevResult = output;
	}
}

//
// Function: remove()
// Called when the widget has been removed from the Dashboard
//
function remove()
{
    // Stop any timers to prevent CPU usage
    // Remove any preferences as needed
    // widget.setPreferenceForKey(null, createInstancePreferenceKey("your-key"));
}

//
// Function: hide()
// Called when the widget has been hidden
//
function hide()
{
    // Stop any timers to prevent CPU usage
}

//
// Function: show()
// Called when the widget has been shown
//
function show()
{
    // Restart any timers that were stopped on hide
}

//
// Function: sync()
// Called when the widget has been synchronized with .Mac
//
function sync()
{
    // Retrieve any preference values that you need to be synchronized here
    // Use this for an instance key's value:
    // instancePreferenceValue = widget.preferenceForKey(null, createInstancePreferenceKey("your-key"));
    //
    // Or this for global key's value:
    // globalPreferenceValue = widget.preferenceForKey(null, "your-key");
}

//
// Function: showBack(event)
// Called when the info button is clicked to show the back of the widget
//
// event: onClick event from the info button
//
function showBack(event)
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        widget.prepareForTransition("ToBack");
    }

    front.style.display = "none";
    back.style.display = "block";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
	
	document.getElementById("magicLetter").value = magicLetter;
}

//
// Function: showFront(event)
// Called when the done button is clicked from the back of the widget
//
// event: onClick event from the done button
//
function showFront(event)
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        widget.prepareForTransition("ToFront");
    }

    front.style.display="block";
    back.style.display="none";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
	
	// save the value for magicLetter (in case they've changed it coming from the back
	if (document.getElementById("magicLetter").value != ""){
		widget.setPreferenceForKey(document.getElementById("magicLetter").value, "magicLetter");
		magicLetter = document.getElementById("magicLetter").value;
	}
}

if (window.widget) {
    widget.onremove = remove;
    widget.onhide = hide;
    widget.onshow = show;
    widget.onsync = sync;
}
